hirom

;FF5 C2 bank sets DBR to $7E
bank $7E

;Set pad byte to fill newly freed space, $EA is NOP
padbyte $EA

;Refactor some of the weapon attack types to reuse sword code
;This frees rom space and adds features like spellblade/double grip support


;Attack type 2F (Harps, was Unused)
;Bells damage + status, for non-gravity harps
;Will deal 0 damage when called from a spell unless the damage asm is also applied.
;Params: 	$57 = Battle Power Override if set
;		$58 = Status Chance + Status Type (High bit set for Status 1, unset for Status 2)
;		$59 = Status
org $C26EB1
HarpsAtk:
	JSR BellsAtk	;Bells routine
	LDA $56		;check for miss
	BNE .End
	JMP $8BF9	;Apply Bow-style Status Effects
.End
	RTS	
		
warnpc $C26EBC		;warn if we overflowed the space available
pad $C26EBC             ;pad the remainder


	
;Attack Type 31 (Swords)
;Swords routine is unchanged for now, but need it to set up labels to jump to
org $C26EE1
SwordsAtk:
	JSR $7C47		;(Hit = 100%, Evade = Evade%)
	JSR $7EBE		;Hit% Determination for physical
	LDA $56		
	BNE $6F19
.Hit		
	JSR $7FFC		;(Sword Damage Formula)
.Row
	JSR $839B		;(Check Back Row Modifications)		
.Command		
	JSR $83BD		;(Check for Command Modifiers)
	JSR $8430		;(Check Double Grip Modifier)
	JSR $8512		;(Check Target Status Effect Modifiers to Physical Damage)
	JSR $8533		;(Check Attacker Status Effect Modifiers to Physical Damage)
.MSword
	JSR $8684		;(Check Magic Sword Modifiers)
.MSwordCleanup
	LDA $61  		;(Magic Sword: Check if target instantly died)
	BNE $6F1D			 
	LDA $56	 		;(Magic Sword: Check if Magic Sword attack failed)
	BNE $6F19		
	LDA $57	 		
	STA $4D   		
	JSR $8734 		;(Check Attack Element Modifiers)
	LDA $56		
	BNE $6F19		
	JSR $8811 		;(Calculate Final Damage)
	JMP $8BCF 		;(Check for Magic Sword Status Effects)
.Miss
	LDA #$80		
	STA $56   		;(Attack Misses, Set $56 to 128)
	RTS 

warnpc $C26F1E					;warn if we overflowed the space available
pad $C26F1E                                     ;pad the remainder (though there should be none)


;Attack Type 33 (Spears)
org $C26F58
SpearsAtk:
	JSR $7C47  		;(Hit = 100%, Evade = Evade%)
	JSR $7EBE  		;(Hit% Determination for physical)
	LDA $56			;(nonzero means attack missed)
	BEQ .Hit		;*inverted branch logic to allow for direct jump vs relative branch so code is easily relocatable
	JMP SwordsAtk_Miss	;*finalizes miss using sword routine
.Hit
	JSR $7FFC  		;(Sword Damage Formula)
	JSR $8452  		;(Check for Jump) *moved this ahead of command modifiers since it saves some repeated code and seems fine going first
	JMP SwordsAtk_Command	;*finish rest of calcs in sword routine

warnpc $C26F84					;warn if we overflowed the space available
pad $C26F84                                     ;pad the remainder


;Attack Type 34 (Axes)
org $C26F84
AxesAtk:
	JSR $7C60  		;(Hit = 1st Parameter, Evade = Evade)
	JSR $7EBE  		;(Hit% Determination for physical)
	LDA $56			;(nonzero means attack missed)
	BEQ .Hit		;*choose where to jump depending if we missed
.Miss
	JMP SwordsAtk_Miss	;*clean up from miss in sword routine
.Hit
	JSR $812B		;(Axes damage formula)
	STZ $57			;*sword routine uses $57 for element, axes use it for hit% and have no element
	JMP SwordsAtk_Row	;*finish rest of calcs in sword routine
	
warnpc $C26FA8					;warn if we overflowed the space available
pad $C26FA8                                     ;pad the remainder


;Attack Type 35 (Bows)
;Unchanged, was reusing code before but not anymore, so commented out
;org $C26FA8
;BowsAtk:
;	JSR $7C6C  	;(Hit = 1st Parameter, Evade = Evade/2)
;	JSR $7EBE  	;(Hit% Determination for physical)
;	LDA $56	
;	BNE $6FC4	;
;	JSR $80D4  	;(Knives Damage Formula)
;	JSR $83BD  	;(Check for Command Modifiers)
;	JSR $8512  	;(Check Target Status Effect Modifiers to Physical Damage)
;	JSR $8533  	;(Check Attacker Status Effect Modifiers to Physical Damage)
;.FinalDmg
;	JSR $8A05  	;(Calculate Magic Final Damage)
;	JMP $8BF9  	;(Apply Bow Status Effects)
;	LDA #$80
;	STA $56
;	RTS 
;
;warnpc $C26FC9
;pad $C26FC9


;Attack Type 37 (Katanas)
org $C26FF9
KatanasAtk:
	JSR $7C47  				;(Hit = 100%, Evade = Evade%)
	JSR $7EBE  				;(Hit% Determination for physical)
	LDA $56
;	BEQ .Hit				;**not enough space for this approach
;	JMP SwordsAtk_Miss			;**use this if code is rearranged later
	BNE AxesAtk_Miss			;*ugly hack that bounces off Axe routine because it's closer
.Hit		
	JSR $7FFC  				;(Sword damage formula)
	JSR $839B  				;(Check Back Row Modifications)
	JSR $83BD  				;(Check for Command Modifiers)
	JSR $8430  				;(Check Double Grip Modifiers to Damage)
	JSR $8512  				;(Check Target Status Effect Modifiers to Physical Damage)
	JSR $8533  				;(Check Attacker Status Effect Modifiers to Physical Damage)
	JSR $8684				;*(Check Magic Sword Modifiers) (+3)
	JSR $87DF  				;(Check for Critical Hit) *Note: Existing Crit code skips crits if Magic Sword is active
	STZ $57					;*sword routine uses $57 for element, katanas use it for crit% and have no element (+2)
						;*fun idea would be to copy strengthened element to here so Air Blade does air damage, but there's no space
	JMP SwordsAtk_MSwordCleanup  		;*finish rest of calcs in sword routine
						; (-5) from removed miss code, should fit
						
warnpc $C27020					;warn if we overflowed the space available
pad $C27020                                     ;pad the remainder (though there should be none)

;Attack Type 39 (Bells)
;This is 2 bytes shorter than default routine but it turns out I didn't need to make changes
;So it's commented out for now	
;	
org $C2703E
BellsAtk:
;	JSR $7DF1  		;(Check for Void to Miss)
;	LDA $56			;
;	BEQ .Hit		;*inverted branch logic to allow for direct jump vs relative branch so code is easily relocatable
;	JMP SwordsAtk_Miss	;*finalizes miss using sword routine
;.Hit
;	JSR $816F  		;(Bells damage formula)
;	JSR $84F3  		;(Target Status Effect Modifiers to Magic Damage)
;	JMP $8A05  		;(Calculate Final Damage)
;				;should be 2 bytes free
;warnpc $C27053
;pad $C27053

;Attack Type 3C (Rune Weapons)
org $C2709D
RunesAttack:
	JSR $7C60  		;(Hit = 1st Parameter, Evade = Evade)
	JSR $7EBE  		;(Hit% Determination for physical)
	LDA $56	
	BEQ .Hit		;*check if we missed
	JMP SwordsAtk_Miss	;*clean up from miss in sword routine (+3 bytes)
.Hit
	JSR $812B  		;(Axes damage formula)
	JSR $8467  		;(Bonus to Attack and M, Costing MP)
	STZ $57			;*sword routine uses $57 for element, Rune uses it for hit% and has no element (+2)
	JMP SwordsAtk_Row	;*finish rest of calcs in sword routine
	
;	JSR $839B  (Back Row Modifications to Damage) 		(-3)
;	JSR $83BD  (Command Modifiers to Damage)		(-3)
;	JSR $8430  (Double Grip Modifiers to Damage)		(-3)
;	JSR $8512  (Target Status Effect Modifiers to Damage)	(-3)
;	JSR $8533  (Attacker Status Effect Modifiers to Damage)	(-3)
;	JMP $8A05  (Calculate Magic Final Damage)		(-3)
;	LDA #$80						(-2)
;	STA $56							(-2)
;	RTS 							(-1)

;should be 18 bytes free

warnpc $C270C4	
pad $C270C4

;Attack type 73 Spears (Strong vs Creature)
org $C27AFA
SpearsCreatureAtk:
	JSR $7C47  		;(Hit = 100, Evade = Evade)
	JSR $7EBE  		;(Hit Determination for physical)
	LDA $56			;
	BEQ .Hit		;*check if we missed
	JMP SwordsAtk_Miss	;*clean up from miss in sword routine (+3 bytes)
.Hit
	JSR $7FFC  		;(Swords damage formula)
	JSR $83BD  		;(Command Modifiers to Damage)
	JSR $8452  		;(Check for Jump)
	JSR $8512  		;(Target Status Effect Modifiers to Damage)
	JSR $8533  		;(Attacker Status Effect Modifiers to Damage)
	JSR $87F9  		;(Check Creature type for Critical Hit)
	STZ $57			;*sword routine uses $57 for element, these use it for Creature Type and have no element (+2)
	JMP SwordsAtk_MSword	;*finish rest of calcs in sword routine
				;(-5) from removed miss code, should fit

warnpc $C27B1E					;warn if we overflowed the space available
pad $C27B1E                                     ;pad the remainder (though there should be none)
				