;Damage formula ASM mods
;Improve Bells/Rods formulas to make them somewhat usable
;Add Sta scaling to Brave Blade to make it compete better with Chicken Knife
;Fix Knives formula Agi scaling without making them overpowered
;Refactored some routines to save space (using a generic Stat*Level sub)
;Includes a new Power Drink fix (one from Algo guide doesn't work with the other changes here)

hirom

;FF5 C2 bank sets DBR to $7E
bank $7E

;Set pad byte to fill newly freed space, $EA is NOP, $00 is BRK
padbyte $EA

;Swords damage formula
;Just added Power Drink fix and refactored a bit for some code reuse
org $C27FFC
SwordsDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	TAX 	        	;
	STX $0E	        	;
	JSR $01BF   		;(Divide by 8)
	LDX #$0000		;
	JSR $007C		;
	REP #$20		;
	CLC 	        	;
	ADC $0E    		;(Battle Power + (0..(Battle Power/8))
	STA $50    		;
	STZ $52			;*added so we can reuse str calc for other weapons, could put constant factor here instead if anything needs to use a different one
.StrCalc
	TDC 	        	;
	SEP #$20		;
	LDA $7BE1  		;(Strength)
	JSR StatTimesLevel	;(Strength * Level)
	JSR $01BB  		;(Divide by 128)
	CLC 	        	;
	ADC #$0002 		;(+2)
	ADC $52			;*this is 0 for swords but may have something stored if this code used by other types
	STA $52	        	;
	TDC 	        	;
	SEP #$20		;
.DefCalc
	LDX $49	        	;
	LDA $202D,X		;(Defense)
	TAX 
	STX $54
	RTS 

;warn if we overflowed the space available and pad the remainder
warnpc $C2803E
pad $C2803E

;Fists Damage Formula
;Just Power Drink fix
;Not refactoring this one yet, could save a few bytes but there's a bunch of branches that would need fixed
org $C2803E
FistsDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	NOP
	NOP
	NOP			;continues as normal from here

;Knives damage formula
;Refactored to save space and create a reusable Stat * Level subroutine
;This also fixes agi bug where it ignored high byte of result
;Added Power Drink fix which also needed some changes to account for possible BP overflow
;Reduced stat scaling to /256 instead of /128 and changes static value to 3 to offset rounding loss
;Goal is to restore bugged str/agi scaling without also making them too good and overshadow str weapons for str classes
org $C280D4		;original sub from $80D4 to $812A
KnivesDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	TAX
	STX $50		
	TDC 		
	TAX 		
	LDA #$03		
	JSR $007C    		;(0..3)
	REP #$21 		;*saves a byte by clearing carry alongside m flag
	ADC $50      		;(Battle Power + (0..3))
	STA $50
	TDC
	SEP #$20
	LDA $7BE1    		;(Strength)
	JSR StatTimesLevel	;(Stat * Level)
	JSR $01BA    		;(Divide by 256) *instead of 128
	CLC 		
	ADC #$0003   		;(+3)  *instead of +2, to offset rounding loss
	STA $0E		
	TDC 		
	SEP #$20		
	LDA $7BE2    		;(Agility)
	JSR StatTimesLevel	;(Stat * Level)
	JSR $01BA    		;(Divide by 256) *instead of 128
	CLC 		
	ADC $0E      		;(Level*Strength)/256 + (Level*Agility)/256 + 3
	STA $52		
	TDC 		
	SEP #$20		
	LDX $49		
	LDA $202D,X  		;(Defense)
	TAX 
 	STX $54
	RTS 			;66 bytes, should be 21 bytes free

StatTimesLevel:
;Params: 8 bit Stat in A
;Returns: 16 bit result in A 
;Note: 	Call in 8 bit mode, returns in 16 bit mode
;	This is strange but allows us to follow original code flow without wasting space
;Uses space inside area used by old knife damage routine
	STA $24	
	LDA $7BE5   		;(Level)
	STA $25	
	JSR $00F1   		;(Stat * Level)
	REP #$20
	LDA $26
	RTS			;15 bytes, should be 6 bytes free

;warn if we overflowed the space available and pad the remainder
warnpc $C2812B
pad $C2812B

;Axes Damage formula
;Power Drink fix and refactoring to free space
org $C2812B			;original from $812B to 816E
AxesDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	TAX 
	STX $0E
	LSR $0E      		;(Battle Power/2)
	LDX #$0000		;
	JSR $007C    		;(0..Battle Power)
	REP #$20		;
	CLC 			
	ADC $0E      		;(Battle Power/2) + (0..Battle Power)
	STA $50			
	TDC 			
	SEP #$20		;
	LDA $7BE1    		;(Strength)
	JSR StatTimesLevel	;(Level * Strength)
	JSR $01BB   		;(Divide by 128)
	CLC 			
	ADC #$0002  		;(+2)
	STA $52			
	TDC 			
	SEP #$20		
	LDX $49			
	LDA $202D,X 		;(Defense)
	JSR $01C0   		;(Defense/4)
	TAX 			;
	STX $54
	RTS 			;should be 14 bytes free

;warn if we overflowed the space available and pad the remainder
warnpc $C2816F
pad $C2816F

;Bells Damage formula
;Increases bell damage from 50-100% base damage to 50-150%
;Also divides mdef by 2
;Goal is to make bells usable and have a purpose
;Overrides Battle Power with Param 1 (for harp magic mod, but applies to bells too if data is changed)
org $C2816F			;original sub from $816F to $81CA
BellsDmg:
	LDA $57			;*param 1						(+2)
	BNE .SkipBP		;*if set, use Param 1 instead of BP			(+2)
	JSR LoadBattlePower	;*load Battle Power including Power Drinks 		(-3)
.SkipBP
	LSR          		;(Battle Power/2)
	TAX 		
	STX $0E		
	ROL			;*recover original battle power including carry bit 	(+1)
	LDX #$0000		;may not be needed?
	JSR $007C    		;(0..(Battle Power))
	REP #$21		;*saves a byte by clearing carry alongside m flag	(-1)
	ADC $0E      		;(Battle Power/2) + (0..(Battle Power))
	STA $50		
	TDC 		
	SEP #$20		
	LDA $7BE2    		;(Agility)
	JSR StatTimesLevel	;(Stat * Level)						(-11)
	JSR $01BB    		;(Divide by 128)
	CLC 		
	ADC #$0002   		;(+2)
	STA $0E		
	TDC 		
	SEP #$20		
	LDA $7BE4    		;(Magic Power)						(-11)
	JSR StatTimesLevel	;(Stat * Level)
	JSR $01BB    		;(Divide by 128)
	CLC 		
	ADC $0E      		;(Level*Agility)/128 + (Level*Magic Power)/128 + 2
	STA $52		
	TDC 		
	SEP #$20		
	LDX $49		
	LDA $202F,X  		;(Magic Defense)
	LSR 			;*divide Mdef by 2					(+1)
	TAX 	
	STX $54
	RTS
				;should be 20 bytes free
;warn if we overflowed the space available and pad the remainder
warnpc $C281CB
pad $C281CB

 
;Rods damage formula
;Changes rod damage from 0-200% base damage to 100-200%
;Goal is to make rods not hit for 0 all the time
org $C281CB		;original sub from $81CB to $8204
RodsDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	TAX	
	STX $0E			
	LDX #$0000		
	JSR $007C    		;(0..Battle Power)
	REP #$21		;*saves a byte by clearing carry alongside m
	ADC $0E			;(Battle Power) + (0..Battle Power)
	STA $50		
	TDC 		
	SEP #$20     		
	LDA $7BE4    		;(Magic Power)
	JSR StatTimesLevel	;(Stat * Level)
	JSR $01BA    		;(Divide by 256)
	CLC 		
	ADC #$0002   		;(+2)
	STA $52		
	TDC 		
	SEP #$20		
	LDX $49		
	LDA $202F,X  		;(Magic Defense)
	TAX 		
	STX $54	
	RTS 	

;warn if we overflowed the space available and pad the remainder
warnpc $C28205
pad $C28205

;Brave Blade Damage formula
;Add a Sta/256 factor to M, reuse sword routine code to save space
org $C28577
BraveDmg:
	JSR LoadBattlePower	;*Load Battle Power including Power Drinks
	SEC 
	SBC $7C75    		;(Attack = Attack - # times escaped)
	BCS .NotZero		;
	TDC 			;(Minimum Attack = 0)
.NotZero
	TAX          		;
	STX $50
	LDA $7BE3    		;(Stamina) 
	JSR StatTimesLevel 	;(Stat * Level) 
	JSR $01BA		;(Divide by 256)
	STA $52			;
	JMP SwordsDmg_StrCalc	;*add str and finish up in sword routine
				;should be 22 bytes free

LoadBattlePower:
;Loads Battle Power including Power Drinks
;Params: 	None (uses offsets from $39 and $32)
;Returns: 	Total Battle Power in A (capped at 255)
	LDA $39
	TAX
	LDA $7A03,X		;(Battle Power)
.DrinkOnly			;Use this label to add Power Drink to a different base BP
	LDX $32			;
..SkipX				;Can skip the X load if not needed
	CLC
	ADC $2079,X		;Unused byte in char data repurposed for Power Drink
	BCC .End		;
	LDA #$FF		;255 cap
.End
	RTS			;17 bytes used

;warn if we overflowed the space available and pad the remainder
warnpc $C285AD
pad $C285AD

;Goblin Punch Damage formula
;nothing wrong with it in vanilla but updated to work with new Power Drink fix
org $C285AD		;from $85AD to $85F0
GoblinDmg:
	LDX $32
	LDA $2044,X				;Left Hand
	JSR LoadBattlePower_DrinkOnly_SkipX	;Add Power Drink and cap at 255
	STA $50
	STZ $51					;Clearing high byte
	LDA $47					;Attacker Index
	CMP #$04				;Check if attacker is a monster
	BCS .Monster 				;Monsters don't need the rest of this	
	LDA $2045,X				;Right Hand
	JSR LoadBattlePower_DrinkOnly_SkipX	;Add Power Drink and cap at 255	
	REP #$21				;16 bit and clear carry
	ADC $50					;Left + Right inlcuding drinks
	STA $50					
	STZ $52					;Setting up to use SwordsDmg routine
	JMP SwordsDmg_StrCalc			;*apply str and finish calculations in swords routine
.Monster			
	LDA $2062,X				;
	TAX 				
	STX $52    				;(M = Monster Attack Multiplier)
	JMP SwordsDmg_DefCalc			;*finish calculations in swords routine

;warn if we overflowed the space available and pad the remainder
warnpc $C285F1
pad $C285F1

;Chicken Knife Damage Formula
;Power Drink fix and refactoring to free space
org $C28626				;original from $8626 to $866C
ChickenDmg:
	LDA $7C75			;Flee Count
	LSR				;divide by 2
	JSR LoadBattlePower_DrinkOnly	;*add Power Drink effects (only)
	TAX 
	STX $50
	LDA $7BE2			;(Agility)
	JSR StatTimesLevel		;(Level * Agility) 
	JSR $01BB    			;(Level * Agility)/128
	CLC          			;
	STA $52      			;
	JMP SwordsDmg_StrCalc		;*add str and finish up in sword routine
					;should be 46 bytes free!
				
;warn if we overflowed the space available and pad the remainder
warnpc $C2866D
pad $C2866D
		
;Increase Attack (Power Drink)
;Changing it from updating $2044 and $2045 which are only used by goblin punch
;Instead it now updates $2079 which seems to be unused and initialized to zero 
org $C29197
IncreaseAtk:
	LDX $49
	CLC 
	LDA $2079,X	;*new memory location for power drinks
	ADC $59
	BCC .NotCapped
	LDA #$FF    	;(Max Attack = 255)
.NotCapped
	STA $2079,X
	RTS		;should be 13 bytes free
			
warnpc $C291B4
pad $C291B4